﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Web;
using Bricksoft.BSIM;
using System.Xml;
using System.IO;
using System.Net.Security;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;

namespace Bricksoft
{
    public partial class frmMSNRobot : Form
    {
        const int YAHOO_STATUS_AVAILABLE = 0;
        const int YAHOO_STATUS_BRB = 1;
        const int YAHOO_STATUS_BUSY = 2;
        const int YAHOO_STATUS_NOTATHOME = 3;
        const int YAHOO_STATUS_NOTATDESK = 4;
        const int YAHOO_STATUS_NOTINOFFICE = 5;
        const int YAHOO_STATUS_ONPHONE = 6;
        const int YAHOO_STATUS_ONVACATION = 7;
        const int YAHOO_STATUS_OUTTOLUNCH = 8;
        const int YAHOO_STATUS_STEPPEDOUT = 9;
        const int YAHOO_STATUS_INVISIBLE = 12;
        const int YAHOO_STATUS_CUSTOM = 99;
        const int YAHOO_STATUS_IDLE = 999;
        const int YAHOO_STATUS_WEBLOGIN = 0x55AA555A;
        const int YAHOO_STATUS_OFFLINE = 0x56AA555A;
        const int YAHOO_STATUS_NOTIFY = 0x16;
        const int YAHOO_SERVICE_Y6_STATUS_UPDATE = 0xC6;
        const int YAHOO_SERVICE_STATUS_15 = 0xF0;

        public IMService im = new IMService();
        public SessionInfo session = null;
        public ArrayList messageQueue = new ArrayList();
        public TreeNode rootNode = null;
        public delegate void MyInvoke(string str);

        private Timer timerKeepAlive;
        private ArrayList yahooStatus = new ArrayList();

        public frmMSNRobot()
        {
            System.Net.ServicePointManager.ServerCertificateValidationCallback += delegate(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
            {
                return true;
            };

            InitializeComponent();

            timerKeepAlive = new Timer();
            timerKeepAlive.Interval = 2000;
            timerKeepAlive.Tick += new EventHandler(timerKeepAliveExecute);
            timerKeepAlive.Start();
        }

        private string StatusAsImageKey(string status)
        {
            switch (status)
            {
                case "HDN":
                    return "invisible";

                case "AWY":
                    return "away";

                case "BSY":
                    return "busy";
            }

            return "available";
        }

        private string getIMType()
        {
            if (rdYahoo.Checked)
                return "yahoo";
            if (rdICQ.Checked)
                return "icq";
            if (rdAIM.Checked)
                return "aim";
            if (rdGTalk.Checked)
                return "jabber";
            return "";
        }

        private void timerKeepAliveExecute(Object sender, EventArgs e)
        {
            timerKeepAlive.Enabled = false;
            try
            {
                if (session != null)
                {
                    messageQueue.Add(im.KeepSessionAlive(session));
                    if (messageQueue.Count > 0)
                        processMessage();
                }
            }
            finally
            {
                timerKeepAlive.Enabled = true;
            }
        }

        private TreeNode findNode(TreeNode tnParent, string strValue)
        {

            if (tnParent == null) return null;

            if ((string)tnParent.Tag == strValue)
                return tnParent;

            TreeNode tnRet = null;

            foreach (TreeNode tn in tnParent.Nodes)
            {
                tnRet = findNode(tn, strValue);
                if (tnRet != null) break;
            }
            return tnRet;
        }


        private void processMessage()
        {
            string responseStr = (string)messageQueue[0];
            messageQueue.RemoveAt(0);

            //showLog(responseXml);

            XmlDocument responseXml = new XmlDocument();
            responseXml.LoadXml(responseStr);

            XmlNode sessionNode = responseXml.SelectSingleNode("Session");

            //IM Account List
            foreach (XmlNode imNode in sessionNode.ChildNodes)
            {
                foreach (XmlNode eventNode in imNode.ChildNodes)
                {
                    switch (eventNode.Name)
                    {    
                        case "OnDeleteBuddy":
                            foreach (XmlNode item in eventNode.ChildNodes)
                            {
                                showLog("OnDeleteBuddy --> " + item.InnerText);

                                TreeNode n = findNode(rootNode, item.Attributes["account"].InnerText);
                                if (n != null) n.Remove();
                            }
                            lblStatus.Text = "OnDeleteBuddy occurred"; 

                            break;

                        case "OnLog":
                            foreach (XmlNode item in eventNode.ChildNodes)
                            {
                                showLog("OnLog --> " + item.InnerText);
                                lblStatus.Text = item.InnerText;
                            }

                            break;

                        case "OnError":
                            if ((tvIM.Nodes.Count <= 0) && (!cmdLogin.Enabled))
                                cmdLogin.Enabled = true;

                            foreach (XmlNode item in eventNode.ChildNodes)
                            {
                                showLog("OnError --> " + item.InnerText);
                                lblStatus.Text = item.InnerText;
                            }
                            lblStatus.Text = "OnError occurred"; 

                            break;

                        case "OnLogin":
                            showLog("OnLogin --> " + eventNode.InnerText);

                            rootNode = new TreeNode(txtIMAccount.Text);

                            cmdLogin.Enabled = false;
                            cbStatus.Enabled = true;
                            mmAction.Enabled = true;

                            tvIM.Nodes.Add(rootNode);
                            rootNode.Text = txtIMAccount.Text;

                            tabControl1.SelectedIndex = 1;

                            lblStatus.Text = "Login OK";

                            switch (getIMType())
                            {
                                case "yahoo":
                                    break;
                                case "icq":
                                    break;
                            }
                            break;

                        case "OnLogout":
                            showLog("OnLogout --> " + eventNode.InnerText);

                            tvIM.Nodes.Clear();

                            cmdLogin.Enabled = true;
                            cbStatus.Enabled = false;
                            mmAction.Enabled = false;

                            tabControl1.SelectedIndex = 0;

                            lblStatus.Text = "Logout"; 

                            break;

                        case "OnReceiveGroups":
                            foreach (XmlNode group in eventNode.ChildNodes)
                            {
                                TreeNode gn = null;
                                switch (getIMType())
                                {
                                    case "yahoo":
                                    case "jabber":
                                        gn = new TreeNode(System.Web.HttpUtility.UrlDecode(group.InnerText, Encoding.UTF8));
                                        rootNode.Nodes.Add(gn);
                                        gn.SelectedImageKey = "group";
                                        gn.ImageKey = "group";
                                        gn.Tag = System.Web.HttpUtility.UrlDecode(group.InnerText, Encoding.UTF8);
                                        break;

                                    case "icq":
                                    case "aim":
                                        gn = new TreeNode(System.Web.HttpUtility.UrlDecode(group.SelectSingleNode("Name").InnerText, Encoding.UTF8));
                                        rootNode.Nodes.Add(gn);
                                        gn.SelectedImageKey = "group";
                                        gn.ImageKey = "group";
                                        gn.Tag = group.SelectSingleNode("Id").InnerText;
                                        break;

                                }
                            }

                            lblStatus.Text = "OnReceiveGroups occurred"; 

                            break;

                        case "OnReceiveBuddies":
                            foreach (XmlNode buddy in eventNode.ChildNodes)
                            {
                                TreeNode gn = null;
                                TreeNode bn = null;
                                switch (getIMType())
                                {
                                    case "yahoo":
                                    case "jabber":
                                        gn = findNode(rootNode, buddy.SelectSingleNode("GroupId").InnerText);

                                        bn = new TreeNode(buddy.SelectSingleNode("Account").InnerText);
                                        bn.Tag = buddy.SelectSingleNode("Account").InnerText;
                                        bn.ImageKey = StatusAsImageKey(buddy.SelectSingleNode("Status").InnerText);
                                        bn.SelectedImageKey = bn.ImageKey;

                                        if (gn == null)
                                            rootNode.Nodes.Add(bn);
                                        else
                                            gn.Nodes.Add(bn);

                                        break;

                                    case "icq":
                                    case "aim":
                                        gn = findNode(rootNode, buddy.SelectSingleNode("GroupId").InnerText);

                                        bn = new TreeNode(buddy.SelectSingleNode("Account").InnerText);
                                        bn.Tag = buddy.SelectSingleNode("Account").InnerText;
                                        bn.ImageKey = StatusAsImageKey(buddy.SelectSingleNode("Status").InnerText);
                                        bn.SelectedImageKey = bn.ImageKey;

                                        if (gn == null)
                                            rootNode.Nodes.Add(bn);
                                        else
                                            gn.Nodes.Add(bn);

                                        break;
                                }
                            }
                            rootNode.ExpandAll();

                            lblStatus.Text = "OnReceiveBuddies occurred"; 

                            break;

                        case "OnAddBuddy":
                            foreach (XmlNode buddy in eventNode.ChildNodes)
                            {
                                switch (getIMType())
                                {
                                    case "yahoo":
                                        TreeNode gn = findNode(rootNode, buddy.SelectSingleNode("GroupId").InnerText);

                                        TreeNode bn = new TreeNode(buddy.SelectSingleNode("Account").InnerText);
                                        bn.Tag = buddy.SelectSingleNode("Account").InnerText;
                                        bn.ImageKey = StatusAsImageKey(buddy.SelectSingleNode("Status").InnerText);
                                        bn.SelectedImageKey = bn.ImageKey;

                                        if (gn == null)
                                            rootNode.Nodes.Add(bn);
                                        else
                                            gn.Nodes.Add(bn);
                                        break;
                                }
                            }
                            rootNode.ExpandAll();

                            lblStatus.Text = "OnAddBuddy occurred";

                            break;

                        case "OnBuddyAddYouRequest":
                            foreach (XmlNode buddy in eventNode.ChildNodes)
                            {
                                if (MessageBox.Show(buddy.Attributes["account"].InnerText + " added you. Do you want " + buddy.Attributes["account"].InnerText + " to see you?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                                    im.AddBuddy(session, getIMType(), txtIMAccount.Text, buddy.Attributes["account"].InnerText);
                            }

                            lblStatus.Text = "OnBuddyAddYouRequest occurred";

                            break;

                        case "OnMessage":
                            foreach (XmlNode message in eventNode.ChildNodes)
                            {
                                showLog("OnMessage --> " + message.Attributes["from"].InnerText + " Said: " +
                                    System.Web.HttpUtility.UrlDecode(message.InnerText, Encoding.UTF8));
                            }

                            lblStatus.Text = "OnMessage occurred"; 

                            break;

                        case "OnOfflineMessage":
                            foreach (XmlNode message in eventNode.ChildNodes)
                            {
                                showLog("OnOfflineMessage --> [" + message.Attributes["datetime"].InnerText + "] " + message.Attributes["from"].InnerText + " Said: " +
                                    System.Web.HttpUtility.UrlDecode(message.InnerText, Encoding.UTF8));
                            }

                            lblStatus.Text = "OnOfflineMessage occurred"; 

                            break;


                        case "OnNicknameChanged":
                            //txtNickname.Text = System.Web.HttpUtility.UrlDecode(eventNode.InnerText, Encoding.UTF8);
                            //rootNode.Text = txtNickname.Text;

                            lblStatus.Text = "OnNicknameChanged occurred"; 

                            break;

                        case "OnBuddyStatusChanged":
                            foreach (XmlNode buddy in eventNode.ChildNodes)
                            {
                                TreeNode buddyNode = findNode(rootNode, buddy.SelectSingleNode("Account").InnerText);

                                if (buddyNode != null)
                                {

                                    buddyNode.ImageKey = StatusAsImageKey(buddy.SelectSingleNode("Status").InnerText);
                                    buddyNode.SelectedImageKey = buddyNode.ImageKey;

                                    if (buddyNode != null)
                                    {
                                        switch (getIMType())
                                        {
                                            case "yahoo":

                                                if ((buddy.SelectSingleNode("Status").InnerText == "CST") && (System.Web.HttpUtility.UrlDecode(buddy.SelectSingleNode("StatusMessage").InnerText, Encoding.UTF8) != "")) //custom status
                                                    buddyNode.Text = buddy.SelectSingleNode("Account").InnerText + " - " + System.Web.HttpUtility.UrlDecode(buddy.SelectSingleNode("StatusMessage").InnerText, Encoding.UTF8);
                                                else
                                                    buddyNode.Text = buddy.SelectSingleNode("Account").InnerText;

                                                break;
                                        }
                                    }
                                }
                            }

                            lblStatus.Text = "OnBuddyStatusChanged occurred"; 

                            break;

                        case "OnStatusChanged":

                            /*
                              Online - NLN
                              Busy - BSY
                              Away - AWY
                              Offline - HDN
                             */

                            switch (eventNode.InnerText)
                            {
                                case "NLN":
                                    cbStatus.SelectedIndex = 0;
                                    break;
                                case "BSY":
                                    cbStatus.SelectedIndex = 1;
                                    break;
                                case "AWY":
                                    cbStatus.SelectedIndex = 2;
                                    break;
                                default:
                                    cbStatus.SelectedIndex = 3;
                                    break;
                            };

                            rootNode.ImageKey = StatusAsImageKey(eventNode.InnerText);
                            rootNode.SelectedImageKey= rootNode.ImageKey;

                            lblStatus.Text = "OnStatusChanged occurred"; 

                            break;

                        default:
                            showLog(eventNode.OuterXml);
                            break;
                    }
                }
            }
        }

        private void showLog(string log)
        {
            if (txtLog.InvokeRequired)
            {
                MyInvoke _myInvoke = new MyInvoke(showLog);
                this.Invoke(_myInvoke, new object[] { log });
            }
            else
                txtLog.Text = "[" + System.DateTime.Now.ToLocalTime() + "] " + log + "\r\n" + txtLog.Text;
        }

        private void fontDialog1_Apply(object sender, EventArgs e)
        {

        }

        private void cmdLogin_Click(object sender, EventArgs e)
        {
            lblStatus.Text = "Connect Web Service...";

            cmdLogin.Enabled = false;

            tvIM.Nodes.Clear();

            if (rdGTalk.Checked) 
            {
                // 'PLAIN 'DIGEST-MD5' 'X-GOOGLE-TOKEN' 'NTLM'
                im.LoginJabber(session,txtIMAccount.Text, txtIMPassword.Text,"gmail.com","5222","talk.google.com","X-GOOGLE-TOKEN");
            }
            else
                im.LoginIM(session, getIMType(), txtIMAccount.Text, txtIMPassword.Text);

        }

        private void toolStripStatusLabel1_Click(object sender, EventArgs e)
        {

        }

        private void cbStatus_SelectedIndexChanged(object sender, EventArgs e)
        {
            /*
              Online - NLN
              Busy - BSY
              Away - AWY
              Offline - HDN
             */
            string status = "";

            switch (cbStatus.SelectedIndex)
            {
                case 0: status = "NLN"; break;
                case 1: status = "BSY"; break;
                case 2: status = "AWY"; break;
                case 3: status = "HDN"; break;
            }
            im.ChangeStatus(session, getIMType(), txtIMAccount.Text, status);
        }

        private void txtNickname_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                //im.ChangeNickname(session, getIMType(), txtIMAccount.Text, HttpUtility.UrlEncode(txtNickname.Text, Encoding.UTF8));
            }
        }

        private void blockToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tvIM.SelectedNode != null)
            {
                im.BlockBuddy(session, getIMType(), txtIMAccount.Text, (string)tvIM.SelectedNode.Tag);
            }

        }

        private void tvMSN_AfterSelect(object sender, TreeViewEventArgs e)
        {
            mmAction.Enabled = !((e.Node == rootNode) || (e.Node.ImageKey == "group"));
        }

        private void unblockToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tvIM.SelectedNode != null)
            {
                im.UnblockBuddy(session, getIMType(), txtIMAccount.Text, (string)tvIM.SelectedNode.Tag);
            }

        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tvIM.SelectedNode != null)
            {
                im.DeleteBuddy(session, getIMType(), txtIMAccount.Text, (string)tvIM.SelectedNode.Tag);
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void addBuddyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //add buddy
            InputMessageForm frmInput = new InputMessageForm();
            frmInput.setCaption("Add Buddy:");
            frmInput.ShowDialog();

            if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                im.AddBuddy(session, getIMType(), txtIMAccount.Text, frmInput.MessageStr);
        }

        private void logoutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            im.LogoutIM(session, getIMType(), txtIMAccount.Text);
        }


        /*private string MSNStatusToImageIndex(MSNStatus status)
        {
            switch (status)
            {
                case MSNStatus.msnIdle:
                case MSNStatus.msnAway:
                case MSNStatus.msnBeRightBack:
                case MSNStatus.msnOutLunch:
                    return "away";

                case MSNStatus.msnOnline:
                    return "online";

                case MSNStatus.msnBusy:
                case MSNStatus.msnOnThePhone:
                    return "busy";

                default:
                    return "offline";
            }
        }*/

        private void sendMessageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tvIM.SelectedNode != null)
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.setCaption("Instant Message:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                {
                    im.SendMessage(session, getIMType(), txtIMAccount.Text, (string)tvIM.SelectedNode.Tag, System.Web.HttpUtility.UrlEncode(frmInput.MessageStr, Encoding.UTF8));
                }
            }
        }

        private void sendOfflineMessageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tvIM.SelectedNode != null)
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.setCaption("Offline Message:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                {
                    im.SendOfflineMessage(session, getIMType(), txtIMAccount.Text, (string)tvIM.SelectedNode.Tag, System.Web.HttpUtility.UrlEncode(frmInput.MessageStr, Encoding.UTF8));
                }
            }
        }

        private void loginToolStripMenuItem_Click(object sender, EventArgs e)
        {
            cmdLogin_Click(sender,e);
        }

        private void sendShakeToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void pbAvatar_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if ((session!=null) && (ofdAvatar.ShowDialog() == DialogResult.OK))
            {
                //msn.AvatarImageFilename = ofdAvatar.FileName;
                //pbAvatar.ImageLocation = ofdAvatar.FileName;
                FileStream fs = new FileStream(ofdAvatar.FileName,FileMode.Open);
                byte[] buff = new byte[fs.Length];
                fs.Read(buff,0,(int)fs.Length);
                fs.Close();
                string base64 = Convert.ToBase64String(buff);
                im.SetDisplayPicture(session, getIMType(), txtIMAccount.Text, base64);
                //msn.BroadcastAvatarImage();
            }
        }

        private void toolStripMenuItem4_Click(object sender, EventArgs e)
        {
        }

        private void button1_Click(object sender, EventArgs e)
        {
        }

        private void cmdLogout_Click(object sender, EventArgs e)
        {
            im.LogoutIM(session, getIMType(), txtIMAccount.Text);
        }

        private void button1_Click_1(object sender, EventArgs e)
        {
        }

        private void frmMSNRobot_Shown(object sender, EventArgs e)
        {
            showLog("Login() -->");

            frmLogin login = new frmLogin();
            login.im = im;
            if (login.ShowDialog() == DialogResult.OK)
            {
                session = login.session;

                showLog("SessionId = " + session.SessionId + "(" + session.ClientIP + ")");

                lblStatus.Text = "Get session OK. You can login IM account now";

                return;
            }

            Close();
        }

        private void menuStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {

        }

        private void toolStripMenuItem5_Click(object sender, EventArgs e)
        {
            FileStream fs = new FileStream("log.txt",FileMode.Create);
            byte[] bytes = Encoding.Default.GetBytes(txtLog.Text);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            MessageBox.Show("Export Log to \"log.txt\".");
        }

        private void frmMSNRobot_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (session != null)
                im.Logout(session);
        }

    }
}