﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Web;
using Bricksoft.BSIM;
using System.Xml;
using System.IO;
using System.Net.Security;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;

namespace Bricksoft
{
    public partial class frmMSNRobot : Form
    {
        public IMService im = new IMService();
        public SessionInfo session = null;
        public ArrayList messageQueue = new ArrayList();
        public TreeNode rootNode = null;
        public delegate void MyInvoke(string str);

        private Timer timerKeepAlive;

        public frmMSNRobot()
        {
            InitializeComponent();

            System.Net.ServicePointManager.ServerCertificateValidationCallback += delegate(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
            {
                return true;
            };

            timerKeepAlive = new Timer();
            timerKeepAlive.Interval = 2000;
            timerKeepAlive.Tick += new EventHandler(timerKeepAliveExecute);
            timerKeepAlive.Start();
        }

        private string getIMType()
        {
            if (rdMSN.Checked)
                return "msn";
            if (rdYahoo.Checked)
                return "yahoo";
            if (rdICQ.Checked)
                return "icq";
            if (rdAIM.Checked)
                return "aim";
            return "";
        }

        private void timerKeepAliveExecute(Object sender, EventArgs e)
        {
            timerKeepAlive.Enabled = false;
            try
            {
                if (session != null)
                {
                    messageQueue.Add(im.KeepSessionAlive(session));
                    if (messageQueue.Count > 0)
                        processMessage();
                }
            }
            finally
            {
                timerKeepAlive.Enabled = true;
            }
        }

        private TreeNode findNode(TreeNode tnParent, string strValue)
        {

            if (tnParent == null) return null;

            if ((string)tnParent.Tag == strValue)
                return tnParent;

            TreeNode tnRet = null;

            foreach (TreeNode tn in tnParent.Nodes)
            {
                tnRet = findNode(tn, strValue);
                if (tnRet != null) break;
            }
            return tnRet;
        }


        private void processMessage()
        {
            string responseStr = (string)messageQueue[0];
            messageQueue.RemoveAt(0);

            //showLog(responseXml);

            XmlDocument responseXml = new XmlDocument();
            responseXml.LoadXml(responseStr);

            XmlNode sessionNode = responseXml.SelectSingleNode("Session");

            //IM Account List
            foreach (XmlNode imNode in sessionNode.ChildNodes)
            {
                foreach (XmlNode eventNode in imNode.ChildNodes)
                {
                    switch (eventNode.Name)
                    {    
                        case "OnDeleteBuddy":
                            foreach (XmlNode item in eventNode.ChildNodes)
                            {
                                showLog("OnDeleteBuddy --> " + item.InnerText);

                                TreeNode n = findNode(rootNode, item.Attributes["account"].InnerText);
                                if (n != null) n.Remove();
                            }
                            lblStatus.Text = "OnDeleteBuddy occurred"; 

                            break;

                        case "OnLog":
                            foreach (XmlNode item in eventNode.ChildNodes)
                            {
                                showLog("OnLog --> " + item.InnerText);
                                lblStatus.Text = item.InnerText;
                            }

                            break;

                        case "OnError":
                            if ((tvIM.Nodes.Count <= 0) && (!cmdLogin.Enabled))
                                cmdLogin.Enabled = true;

                            foreach (XmlNode item in eventNode.ChildNodes)
                            {
                                showLog("OnError --> " + item.InnerText);
                                lblStatus.Text = item.InnerText;
                            }
                            lblStatus.Text = "OnError occurred"; 

                            break;

                        case "OnLogin":
                            showLog("OnLogin --> " + eventNode.InnerText);

                            rootNode = new TreeNode(txtIMAccount.Text);

                            cmdLogin.Enabled = false;
                            cbStatus.Enabled = true;
                            txtNickname.Enabled = true;
                            mmAction.Enabled = true;
                            pbAvatar.Enabled = true;

                            tvIM.Nodes.Add(rootNode);
                            rootNode.Text = txtIMAccount.Text;

                            tabControl1.SelectedIndex = 1;

                            lblStatus.Text = "Login OK"; 
                            break;

                        case "OnLogout":
                            showLog("OnLogout --> " + eventNode.InnerText);

                            tvIM.Nodes.Clear();

                            txtNickname.Enabled = false;
                            cmdLogin.Enabled = true;
                            cbStatus.Enabled = false;
                            mmAction.Enabled = false;
                            pbAvatar.Enabled = false;

                            tabControl1.SelectedIndex = 0;

                            lblStatus.Text = "Logout"; 

                            break;

                        case "OnReceiveGroups":
                            foreach (XmlNode group in eventNode.ChildNodes)
                            {
                                TreeNode gn = new TreeNode(System.Web.HttpUtility.UrlDecode(group.SelectSingleNode("Name").InnerText, Encoding.UTF8));
                                rootNode.Nodes.Add(gn);
                                gn.SelectedImageKey = "group";
                                gn.ImageKey = "group";
                                gn.Tag = group.SelectSingleNode("Id").InnerText;
                                break;
                            }

                            lblStatus.Text = "OnReceiveGroups occurred"; 

                            break;

                        case "OnReceiveBuddies":
                            foreach (XmlNode buddy in eventNode.ChildNodes)
                            {
                                TreeNode gn = findNode(rootNode, buddy.SelectSingleNode("GroupId").InnerText);

                                TreeNode bn = new TreeNode(System.Web.HttpUtility.UrlDecode(buddy.SelectSingleNode("Nickname").InnerText, Encoding.UTF8));
                                bn.Tag = buddy.SelectSingleNode("Account").InnerText;

                                if (gn == null)
                                    rootNode.Nodes.Add(bn);
                                else
                                    gn.Nodes.Add(bn);
                            }
                            rootNode.ExpandAll();

                            lblStatus.Text = "OnReceiveBuddies occurred"; 

                            break;

                        case "OnAddBuddy":
                            foreach (XmlNode buddy in eventNode.ChildNodes)
                            {
                                TreeNode gn = findNode(rootNode, buddy.SelectSingleNode("GroupId").InnerText);

                                TreeNode bn = new TreeNode(System.Web.HttpUtility.UrlDecode(buddy.SelectSingleNode("Nickname").InnerText, Encoding.UTF8));
                                bn.Tag = buddy.SelectSingleNode("Account").InnerText;

                                if (gn == null)
                                    rootNode.Nodes.Add(bn);
                                else
                                    gn.Nodes.Add(bn);
                            }
                            rootNode.ExpandAll();

                            lblStatus.Text = "OnAddBuddy occurred";

                            break;

                        case "OnBuddyAddYouRequest":
                            foreach (XmlNode buddy in eventNode.ChildNodes)
                            {
                                if (MessageBox.Show(buddy.Attributes["account"].InnerText + " added you. Do you want " + buddy.Attributes["account"].InnerText + " to see you?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                                    im.AddBuddy(session, getIMType(), txtIMAccount.Text, buddy.Attributes["account"].InnerText);
                            }

                            lblStatus.Text = "OnBuddyAddYouRequest occurred";

                            break;

                        case "OnMessage":
                            foreach (XmlNode message in eventNode.ChildNodes)
                            {
                                showLog("OnMessage --> " + message.Attributes["from"].InnerText + " Said: " +
                                    System.Web.HttpUtility.UrlDecode(message.InnerText, Encoding.UTF8));
                            }

                            lblStatus.Text = "OnMessage occurred"; 

                            break;

                        case "OnOfflineMessage":
                            foreach (XmlNode message in eventNode.ChildNodes)
                            {
                                showLog("OnOfflineMessage --> [" + message.Attributes["datetime"].InnerText + "] " + message.Attributes["from"].InnerText + " Said: " +
                                    System.Web.HttpUtility.UrlDecode(message.InnerText, Encoding.UTF8));
                            }

                            lblStatus.Text = "OnOfflineMessage occurred"; 

                            break;


                        case "OnNicknameChanged":
                            txtNickname.Text = System.Web.HttpUtility.UrlDecode(eventNode.InnerText, Encoding.UTF8);
                            rootNode.Text = txtNickname.Text;

                            lblStatus.Text = "OnNicknameChanged occurred"; 

                            break;

                        case "OnBuddyStatusChanged":
                            foreach (XmlNode item in eventNode.ChildNodes)
                            {
                                TreeNode buddyNode = findNode(rootNode, item.SelectSingleNode("Account").InnerText);

                                if (buddyNode != null)
                                {
                                    if (item.SelectSingleNode("Blocked").InnerText == "1")
                                    {
                                        if (item.SelectSingleNode("Status").InnerText == "HDN")
                                        {
                                            buddyNode.ImageKey = "offlineblock";
                                            buddyNode.SelectedImageKey = "offlineblock";
                                        }
                                        else
                                        {
                                            buddyNode.ImageKey = "onlineblock";
                                            buddyNode.SelectedImageKey = "onlineblock";
                                        }
                                    }
                                    else
                                    {
                                        buddyNode.ImageKey = MSNStatusToImageKey(item.SelectSingleNode("Status").InnerText);
                                        buddyNode.SelectedImageKey = buddyNode.ImageKey;
                                    }

                                    if (item.SelectSingleNode("PersonalMessage").InnerText != "")
                                        buddyNode.Text = HttpUtility.UrlDecode(item.SelectSingleNode("Nickname").InnerText, Encoding.UTF8) + " - " + HttpUtility.UrlDecode(item.SelectSingleNode("PersonalMessage").InnerText, Encoding.UTF8);
                                    else
                                        buddyNode.Text = HttpUtility.UrlDecode(item.SelectSingleNode("Nickname").InnerText, Encoding.UTF8);
                                }
                            }

                            lblStatus.Text = "OnBuddyStatusChanged occurred"; 

                            break;

                        case "OnStatusChanged":
                            rootNode.ImageKey = MSNStatusToImageKey(eventNode.InnerText);
                            rootNode.SelectedImageKey = rootNode.ImageKey;


                            switch (eventNode.InnerText)
                            {
                                case "IDL":
                                    cbStatus.SelectedIndex = 3;
                                    break;
                                case "AWY":
                                    cbStatus.SelectedIndex = 4;
                                    break;
                                case "BRB":
                                    cbStatus.SelectedIndex = 2;
                                    break;
                                case "LUN":
                                    cbStatus.SelectedIndex = 6;
                                    break;
                                case "NLN":
                                    cbStatus.SelectedIndex = 0;
                                    break;
                                case "BSY":
                                    cbStatus.SelectedIndex = 1;
                                    break;
                                case "PHN":
                                    cbStatus.SelectedIndex = 5;
                                    break;

                                default:
                                    cbStatus.SelectedIndex = 7;
                                    break;
                            }

                            lblStatus.Text = "OnStatusChanged occurred"; 

                            break;

                        default:
                            showLog(eventNode.OuterXml);
                            break;
                    }
                }
            }
        }

        private string MSNStatusToImageKey(string status)
        {
            switch (status)
            {
                case "IDL":
                case "AWY":
                case "BRB":
                case "LUN":
                    return "away";

                case "NLN":
                    return "online";

                case "BSY":
                case "PHN":
                    return "busy";

                default:
                    return "offline";
            }
        }


        private void showLog(string log)
        {
            if (txtLog.InvokeRequired)
            {
                MyInvoke _myInvoke = new MyInvoke(showLog);
                this.Invoke(_myInvoke, new object[] { log });
            }
            else
                txtLog.Text = "[" + System.DateTime.Now.ToLocalTime() + "] " + log + "\r\n" + txtLog.Text;
        }

        private void fontDialog1_Apply(object sender, EventArgs e)
        {

        }

        private void cmdLogin_Click(object sender, EventArgs e)
        {
            lblStatus.Text = "Connect Web Service...";

            cmdLogin.Enabled = false;

            tvIM.Nodes.Clear();

            im.LoginIM(session, getIMType(), txtIMAccount.Text, txtIMPassword.Text);
        }

        private void toolStripStatusLabel1_Click(object sender, EventArgs e)
        {

        }

        private void cbStatus_SelectedIndexChanged(object sender, EventArgs e)
        {
            string v = "";
            switch (cbStatus.SelectedIndex)
            {
                case 0:
                    v = "NLN";
                    break;
                case 1:
                    v = "BSY";
                    break;
                case 2:
                    v = "BRB";
                    break;
                case 3:
                    v = "IDL";
                    break;
                case 4:
                    v = "AWY";
                    break;
                case 5:
                    v = "PHN";
                    break;
                case 6:
                    v = "LUN";
                    break;
                case 7:
                    v = "HDN";
                    break;
            }

            im.ChangeStatus(session, getIMType(), txtIMAccount.Text,v);        
        }

        private void txtNickname_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                im.ChangeNickname(session, getIMType(), txtIMAccount.Text, HttpUtility.UrlEncode(txtNickname.Text, Encoding.UTF8));
            }
        }

        private void blockToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tvIM.SelectedNode != null)
            {
                im.BlockBuddy(session, getIMType(), txtIMAccount.Text, (string)tvIM.SelectedNode.Tag);
            }

        }

        private void tvMSN_AfterSelect(object sender, TreeViewEventArgs e)
        {
        }

        private void unblockToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tvIM.SelectedNode != null)
            {
                im.UnblockBuddy(session, getIMType(), txtIMAccount.Text, (string)tvIM.SelectedNode.Tag);
            }

        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tvIM.SelectedNode != null)
            {
                im.DeleteBuddy(session, getIMType(), txtIMAccount.Text, (string)tvIM.SelectedNode.Tag);
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void addBuddyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //add buddy
            InputMessageForm frmInput = new InputMessageForm();
            frmInput.setCaption("Add Buddy:");
            frmInput.ShowDialog();

            if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                im.AddBuddy(session, getIMType(), txtIMAccount.Text, frmInput.MessageStr);
        }

        private void logoutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            im.LogoutIM(session, getIMType(), txtIMAccount.Text);
        }


        /*private string MSNStatusToImageIndex(MSNStatus status)
        {
            switch (status)
            {
                case MSNStatus.msnIdle:
                case MSNStatus.msnAway:
                case MSNStatus.msnBeRightBack:
                case MSNStatus.msnOutLunch:
                    return "away";

                case MSNStatus.msnOnline:
                    return "online";

                case MSNStatus.msnBusy:
                case MSNStatus.msnOnThePhone:
                    return "busy";

                default:
                    return "offline";
            }
        }*/

        private void sendMessageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tvIM.SelectedNode != null)
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.setCaption("Instant Message:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                {
                    im.SendMessage(session, getIMType(), txtIMAccount.Text, (string)tvIM.SelectedNode.Tag, System.Web.HttpUtility.UrlEncode(frmInput.MessageStr, Encoding.UTF8));
                }
            }
        }

        private void sendOfflineMessageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tvIM.SelectedNode != null)
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.setCaption("Offline Message:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                {
                    im.SendOfflineMessage(session, getIMType(), txtIMAccount.Text, (string)tvIM.SelectedNode.Tag, System.Web.HttpUtility.UrlEncode(frmInput.MessageStr, Encoding.UTF8));
                }
            }
        }

        private void loginToolStripMenuItem_Click(object sender, EventArgs e)
        {
            cmdLogin_Click(sender,e);
        }

        private void sendShakeToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void pbAvatar_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if ((session!=null) && (ofdAvatar.ShowDialog() == DialogResult.OK))
            {
                //msn.AvatarImageFilename = ofdAvatar.FileName;
                pbAvatar.ImageLocation = ofdAvatar.FileName;
                FileStream fs = new FileStream(ofdAvatar.FileName,FileMode.Open);
                byte[] buff = new byte[fs.Length];
                fs.Read(buff,0,(int)fs.Length);
                fs.Close();
                string base64 = Convert.ToBase64String(buff);
                im.SetDisplayPicture(session, getIMType(), txtIMAccount.Text, base64);
                //msn.BroadcastAvatarImage();
            }
        }

        private void toolStripMenuItem4_Click(object sender, EventArgs e)
        {
        }

        private void button1_Click(object sender, EventArgs e)
        {
        }

        private void cmdLogout_Click(object sender, EventArgs e)
        {
            im.LogoutIM(session, getIMType(), txtIMAccount.Text);
        }

        private void button1_Click_1(object sender, EventArgs e)
        {
        }

        private void frmMSNRobot_Shown(object sender, EventArgs e)
        {
            showLog("Login() -->");

            frmLogin login = new frmLogin();
            login.im = im;
            if (login.ShowDialog() == DialogResult.OK)
            {
                session = login.session;

                showLog("SessionId = " + session.SessionId + "(" + session.ClientIP + ")");

                lblStatus.Text = "Get session OK. You can login IM account now";

                return;
            }

            Close();
        }

        private void menuStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {

        }

        private void toolStripMenuItem5_Click(object sender, EventArgs e)
        {
            FileStream fs = new FileStream("log.txt",FileMode.Create);
            byte[] bytes = Encoding.Default.GetBytes(txtLog.Text);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            MessageBox.Show("Export Log to \"log.txt\".");
        }

        private void frmMSNRobot_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (session != null)
                im.Logout(session);
        }

        private void toolStripMenuItem4_Click_1(object sender, EventArgs e)
        {
            showLog("OnListIM() -> " + im.ListIM(session));
        }

    }
}